==> arirang-2.02.tar.gz: arirang-2.02/ariruby.c <==
/*
 * Copyright 2010, 2011 pilot <pilot@monkey.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>

#include <ruby.h>
#include "arirang.h"


extern int isnetwork;
extern int pflags;
extern int sslflags;
extern short int CONNECT_TIMEOUT;
extern short int RECV_TIMEOUT;
extern int scan_count;

extern int ruby;
extern char *ariruby;
extern FILE *rubyfd;
extern VALUE rb_cArirang;
extern VALUE rb_arirang;


extern int proxy;
extern char *proxyhost;
extern int proxyport;
char rscanhost[128];
int rport;

static int sock;
int nb = 0;
struct sockaddr_in http;
struct hostent *he;
char *sendbuf = NULL;
char *headbuf = NULL;
char *printbuf = NULL;
int recv_size = 2048;
int recv_flag = 0;

SSL_CTX *ctx = NULL;
SSL *ssl = NULL;
int err;
int sread = 0;
int conerr = 0;


void 
breakint(int sig)
{
    exit(0);
}

void
connout_r(int sig)
{
    if (pflags) 
        return;
    exit(0);
}


int
recvtimeout_r(int s, char *buf, int len, int timeout)
{
    fd_set  fds;
    int n;
    struct timeval  tv;

    FD_ZERO(&fds);
    FD_SET(s, &fds);
    tv.tv_sec = timeout;
    tv.tv_usec = 0;

    n = select(s + 1, &fds, NULL, NULL, &tv);
    
    if (n == 0)
        return -2;
    if (n == -1)
        return -1;

#ifdef DEBUG
    printf("recv_flag = %d \n", recv_flag);
#endif

    return recv(s, buf, len, recv_flag);
}


void
ariruby_init(char *rubyfile)
{
    ruby_init();
    ruby_init_loadpath();
    rb_load_file(rubyfile);
}

void
ariruby_run(void)
{
    int status;
    status = ruby_exec();
#ifdef DEBUG
    printf("status  = %d \n", status);
#endif
    if (status != 0) {
        perror("arirang script syntax error\n");
        exit(0);
    }
}

void
ariruby_exit(void)
{
    ruby_finalize();
}


static VALUE 
ariconnect(VALUE self) 
{

    if (proxy) {
        sock = proxy_connect(rscanhost, rport, proxyhost, proxyport, 1);
    }


    if (!proxy) {
        if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
            perror("socket");
            close(sock);
            exit(0);
        }

        signal(SIGALRM, connout_r);
        alarm(CONNECT_TIMEOUT);
        if (connect(sock, (struct sockaddr *) & http, sizeof(struct sockaddr)) == -1) {
            close(sock);
            if (!pflags)
                exit(0);
        }
        alarm(0); 
    }

    if (sslflags) {
        ctx = ssl_init_ctx();
        ssl = ssl_init_ssl(ctx, sock);
    }
#ifdef DEBUG
    printf("connect \n");
#endif

   return self;
}

static VALUE
ariprint(VALUE self, VALUE outbuf)
{
    fprintf(stdout, "%s", RSTRING(outbuf)->ptr);

    return self;    
}

static VALUE 
arisend(VALUE self, VALUE data) 
{
    int len;
    len = RSTRING(data)->len;

    if ((sendbuf = (char *)malloc(len + 1)) == NULL)  {
        perror("arisend malloc failed");
        close(sock);
        exit(0);
    }
    memcpy(sendbuf, RSTRING(data)->ptr, len);

    if (len == 0) {
        fprintf(stderr, "please set a data in arisend function \n");
        exit(0);
    }
#ifdef DEBUG
    printf("send data  = %s\n", sendbuf);
    printf("recv data size = %d \n", recv_size);
#endif

    if ((headbuf = (char *)malloc(recv_size + 1)) == NULL)  {
        perror("malloc failed");
        close(sock);
        exit(0);
    }

    bzero(headbuf, recv_size);

    if (sslflags) {
        err = SSL_write(ssl, sendbuf, len);
        if (err <= 0) {
        close(sock);
/*        fprintf(stdout, "%s - ssl write time out \n", scanhost); */
        exit(0);
        }
#ifdef DEBUG
     printf("error = %d\n", err);
#endif
     sread = ssl_read_timeout(sock, ssl, headbuf, recv_size, RECV_TIMEOUT);

        if (sread <= 0 ) {
            close(sock);
/*            fprintf(stdout, "%s - ssl read time out \n", scanhost); */
        exit(0);
        }
#ifdef DEBUG
     printf("sread  = %d \n", sread);
#endif
    }
    else {
    send(sock, sendbuf, len, 0);
    nb = recvtimeout_r(sock, headbuf, recv_size, RECV_TIMEOUT);
    }

    return (rb_str_new(headbuf, recv_size));
}

static VALUE
ariclose(VALUE self)
{
    close(sock);
    return self;    
}


void
rubyscan(char *scanhost, unsigned int port, short int option)
{

    ID ari_send_ID;
    ID arirang_ID;

    VALUE ari_port;
    VALUE ari_recv;
    VALUE ari_recvflag;
    VALUE rhost;
    VALUE ari_sport;
    char destport[5];
   
    if (!proxy) {
        if ((he = gethostbyname(scanhost)) == NULL) {
            perror("host not found");
            /* close(sock);  */
            exit(0);
        }
    }
    signal(SIGINT, breakint);
    /*ariruby_init(ariruby); */

    arirang_ID = rb_intern("Arirang");
    ari_send_ID = rb_intern("asadal");
    /*ariruby_run(); */

    rb_cArirang = rb_const_get(rb_cObject, arirang_ID);
    rb_arirang = rb_class_new_instance(0, NULL, rb_cArirang);
  
    rb_define_method(rb_cArirang, "ariconnect", ariconnect, 0);
    rb_define_method(rb_cArirang, "arisend", arisend, 1);
    rb_define_method(rb_cArirang, "ariclose", ariclose, 0);
    rb_define_method(rb_cArirang, "ariprint", ariprint, 1);

    ari_port = rb_ivar_get(rb_cArirang, rb_intern("@ari_port"));
    ari_recv = rb_ivar_get(rb_cArirang, rb_intern("@ari_recvsize"));
    ari_recvflag = rb_ivar_get(rb_cArirang, rb_intern("@ari_recvflag"));

/*
    ari_hash = rb_hash_aref(ari_info, rb_str_new2("description"));
    ari_array = rb_ary_entry(ari_info, 0);
*/

    if (ari_port != Qnil)
        port = NUM2INT(ari_port);

    if (proxy) {
        snprintf(rscanhost, sizeof(rscanhost), "%s", scanhost);
        rport = port;
#ifdef DEBUG
        printf("rscanhost = %s, rport = %d \n", rscanhost, rport);
#endif
    }

    if (ari_recv != Qnil)  
        recv_size = NUM2INT(ari_recv);

    if (ari_recvflag != Qnil)
        recv_flag = NUM2INT(ari_recvflag);

    switch (recv_flag) {
        case 1:
            recv_flag = MSG_OOB;
            break;
        case 2:
            recv_flag = MSG_PEEK;
            break;
        case 3:
            recv_flag = MSG_WAITALL;
            break;
        case 4:
            recv_flag = MSG_DONTWAIT;
            break;
        default:
            recv_flag = 0;
    }

    if (!proxy) {
        http.sin_family = AF_INET;
        http.sin_port = htons(port);
        http.sin_addr = *((struct in_addr *) he->h_addr);
        bzero(&(http.sin_zero), 8);
    }

    rhost = rb_str_new2(scanhost);
    rb_define_variable("$ari_host", &rhost);
    snprintf(destport, sizeof(destport), "%d", port);
    ari_sport = rb_str_new2(destport);
    rb_define_variable("$ari_sport", &ari_sport);

    rb_funcall(rb_arirang, ari_send_ID, 0);

    if (sslflags) {
        if (err <= 0) {
        close(sock);
        fprintf(stdout, "%s - ssl write time out \n", scanhost);
        exit(0);
        }

    ssl_error(ssl, err, scanhost);
#ifdef DEBUG
    printf("error = %d\n", err);
#endif
    }

    if (sslflags) {
        if (sread <= 0 ) {
            fprintf(stdout, "%s - ssl read time out \n", scanhost);
            close(sock);
            exit(0);
        }
    }
   
    if (sendbuf != NULL)
        free(sendbuf);

    if (headbuf != NULL)
        free(headbuf);

    if (nb == -1 || nb == -2 ) {
        if (nb == -2)
            fprintf(stdout, "%s - %d/tcp - open port but no receive \n", scanhost, port); 

        close(sock);
        if (!pflags)
           exit(0);
    }

    if (sslflags)
        ssl_free(ctx, ssl);

    ariruby_exit();

    close(sock);

}


void
rubyscanW(unsigned long host, unsigned int port, unsigned short int type, short int option)
{

    ID ari_send_ID;
    ID arirang_ID;

    VALUE ari_port;
    VALUE ari_recv;
    VALUE ari_recvflag;
    VALUE rhost;
    VALUE ari_sport;
    char destport[5];

    signal(SIGINT, breakint);

    /*ariruby_init(ariruby);*/

    arirang_ID = rb_intern("Arirang");
    ari_send_ID = rb_intern( "asadal" );

    /*ariruby_run();*/

    rb_cArirang = rb_const_get(rb_cObject, arirang_ID);
    rb_arirang = rb_class_new_instance(0, NULL, rb_cArirang);

    rb_define_method(rb_cArirang, "ariconnect", ariconnect, 0);
    rb_define_method(rb_cArirang, "arisend", arisend, 1);
    rb_define_method(rb_cArirang, "ariclose", ariclose, 0);
    rb_define_method(rb_cArirang, "ariprint", ariprint, 1);

    ari_port = rb_ivar_get(rb_cArirang, rb_intern("@ari_port"));
    ari_recv = rb_ivar_get(rb_cArirang, rb_intern("@ari_recvsize"));
    ari_recvflag = rb_ivar_get(rb_cArirang, rb_intern("@ari_recvflag"));
/*
    ari_hash = rb_hash_aref(ari_info, rb_str_new2("description"));
    ari_array = rb_ary_entry(ari_info, 0);
*/
    
    if (ari_port != Qnil)
        port = NUM2INT(ari_port);

    if (ari_recv != Qnil)  
        recv_size = NUM2INT(ari_recv);

    if (ari_recvflag != Qnil)
        recv_flag = NUM2INT(ari_recvflag);

    switch (recv_flag) {
        case 1:
            recv_flag = MSG_OOB;
            break;
        case 2:
            recv_flag = MSG_PEEK;
            break;
        case 3:
            recv_flag = MSG_WAITALL;
            break;
        case 4:
            recv_flag = MSG_DONTWAIT;
            break;
        default:
            recv_flag = 0;
    }

    http.sin_family = AF_INET;
    http.sin_port = htons(port);
    http.sin_addr.s_addr = htonl(host);


    if (proxy) {
        snprintf(rscanhost, sizeof(rscanhost), "%s", inet_ntoa(http.sin_addr));
        rport = port;
#ifdef DEBUG
    printf("rscanhost = %s, rport = %d \n", rscanhost, rport);
#endif
    }


#ifdef DEBUG
    printf("Debug %s\n", inet_ntoa(http.sin_addr)); 
#endif

    rhost = rb_str_new2(inet_ntoa(http.sin_addr));
    rb_define_variable("$ari_host", &rhost);
    snprintf(destport, sizeof(destport), "%d", port);
    ari_sport = rb_str_new2(destport);
    rb_define_variable("$ari_sport", &ari_sport);

    rb_funcall(rb_arirang, ari_send_ID, 0);

    if (sslflags) {
        if (err <= 0) {
        close(sock);
        fprintf(stdout, "%s - ssl write time out \n", inet_ntoa(http.sin_addr));
        exit(0);
        }

    ssl_error(ssl, err, inet_ntoa(http.sin_addr)); 
#ifdef DEBUG
    printf("error = %d\n", err); 
#endif
    }

    if (sslflags) {
        if (sread <= 0 ) {
            fprintf(stdout, "%s - ssl read time out \n", inet_ntoa(http.sin_addr));
            close(sock);
            exit(0);
        }
    }
#ifdef DEBUG
    printf("sread  = %d, buf size= %d \n", sread, strlen(headbuf)); 
#endif

    if (sendbuf != NULL)
        free(sendbuf);

    if (headbuf != NULL)
        free(headbuf);

    if (nb == -1 || nb == -2) { 
#if !(defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__))
        alarm(0);
#endif
        if (nb == -2)
            fprintf(stdout, "%s - %d/tcp - open port but no receive \n", inet_ntoa(http.sin_addr), port);
    
        close(sock);
        if (!pflags)
            exit(0);
    }

    if (strstr(headbuf, "Server Error") != NULL) {
#if !(defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__))
        alarm(0);
#endif
        fprintf(stdout, "%s - %d/tcp - server error \n", inet_ntoa(http.sin_addr), port);
        close(sock);
        if (!pflags)
            exit(0);
    }

    if (sslflags)
        ssl_free(ctx, ssl);

    ariruby_exit();

    close(sock);
}

==> arirang-2.03.tar.gz: arirang-2.03/ariruby.c <==
/*
 * Copyright 2010, 2011 pilot <pilot@monkey.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>

#include <ruby.h>
#include "arirang.h"


extern int isnetwork;
extern int pflags;
extern int sslflags;
extern short int CONNECT_TIMEOUT;
extern short int RECV_TIMEOUT;
extern int scan_count;

extern int ruby;
extern char *ariruby;
extern FILE *rubyfd;
extern VALUE rb_cArirang;
extern VALUE rb_arirang;
void *run_node;
char *options[2];

/* ruby 1.8x and 1.9x compat code */
#ifndef RUBY_19
#define RCLASS_M_TBL(c) (RCLASS(c)->m_tbl)
#define RCLASS_SUPER(c) (RCLASS(c)->super)
#define RCLASS_IV_TBL(c) (RCLASS(c)->iv_tbl)
#define RSTRING_PTR(str) (RSTRING(str)->ptr)
#define RSTRING_LEN(str) (RSTRING(str)->len)
#endif


extern int proxy;
extern char *proxyhost;
extern int proxyport;
char rscanhost[128];
int rport;

static int sock;
int nb = 0;
struct sockaddr_in http;
struct hostent *he;
char *sendbuf = NULL;
char *headbuf = NULL;
char *printbuf = NULL;
int recv_size = 2048;
int recv_flag = 0;

SSL_CTX *ctx = NULL;
SSL *ssl = NULL;
int err;
int sread = 0;
int conerr = 0;


void 
breakint(int sig)
{
    DPRINTF("[script] interrupt \n");
    exit(0);
}

void
connout_r(int sig)
{
    DPRINTF("[script] signal connect timeout - %s\n", rscanhost);
    if (pflags) 
        return;
    
    exit(0);
}


int
recvtimeout_r(int s, char *buf, int len, int timeout)
{
    fd_set  fds;
    int n;
    struct timeval  tv;

    FD_ZERO(&fds);
    FD_SET(s, &fds);
    tv.tv_sec = timeout;
    tv.tv_usec = 0;

    n = select(s + 1, &fds, NULL, NULL, &tv);
    
    if (n == 0)
        return -2;
    if (n == -1)
        return -1;

    DPRINTF("[script] %s - @ari_recvflag = %d \n", rscanhost, recv_flag);

    return recv(s, buf, len, recv_flag);
}


void
ariruby_init(char *rubyfile)
{
#ifdef RUBY_19
    DPRINTF("[script] - ruby 1.9x version \n"); 
    RUBY_INIT_STACK;
    options[0] = "";
    options[1] = rubyfile;
#endif
    ruby_init();

#ifndef RUBY_19
    ruby_init_loadpath();
    rb_load_file(rubyfile);
#endif

}

void
ariruby_run(void)
{
    int status;
#ifdef RUBY_19
    /* we use the ruby_exec_node because ruby_run_node crash */ 
    status = ruby_exec_node(ruby_options(2, options));
#else
    status = ruby_exec();
#endif
    DPRINTF("[script] ruby status  = %d \n", status);

    if (status != 0) {
        perror("arirang script syntax error\n");
        exit(0);
    }
}

void
ariruby_exit(void)
{
#ifdef RUBY_19
    ruby_cleanup(0);
#else
    ruby_finalize();
#endif
}


static VALUE 
ariconnect(VALUE self) 
{
    if (proxy) {
        sock = proxy_connect(rscanhost, rport, proxyhost, proxyport, 1);
    }

    if (!proxy) {
        if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
            perror("socket");
            close(sock);
            exit(0);
        }

        signal(SIGALRM, connout_r);
        alarm(CONNECT_TIMEOUT);
        if (connect(sock, (struct sockaddr *) & http, sizeof(struct sockaddr)) == -1) {
			DPRINTF("[script] %s - socket connect failed \n", rscanhost);
            close(sock);
            if (!pflags)
                exit(0);
        }
        alarm(0); 
    }

    if (sslflags) {
        ctx = ssl_init_ctx();
        ssl = ssl_init_ssl(ctx, sock);
    }
    DPRINTF("[script] %s - ariconnect \n", rscanhost);

   return self;
}

static VALUE
ariprint(VALUE self, VALUE outbuf)
{
    fprintf(stdout, "%s", RSTRING_PTR(outbuf));

    return self;    
}

static VALUE 
arisend(VALUE self, VALUE data) 
{
    int len;
    len = RSTRING_LEN(data);

    DPRINTF("[script] %s - arisend \n", rscanhost);
    if ((sendbuf = (char *)malloc(len + 1)) == NULL)  {
        perror("arisend malloc failed");
        close(sock);
        exit(0);
    }
    memcpy(sendbuf, RSTRING_PTR(data), len);

    if (len == 0) {
        fprintf(stderr, "please set a data in arisend function \n");
        exit(0);
    }
    DPRINTF("[script] %s - arisend data  = %s\n", rscanhost, sendbuf);
    DPRINTF("[script] %s - @ari_recvsize = %d \n", rscanhost, recv_size);

    if ((headbuf = (char *)malloc(recv_size + 1)) == NULL)  {
        perror("malloc failed");
        close(sock);
        exit(0);
    }

    bzero(headbuf, recv_size);

    if (sslflags) {
        err = SSL_write(ssl, sendbuf, len);
        if (err <= 0) {
        close(sock);
/*      fprintf(stdout, "%s - ssl write time out \n", scanhost); */
        exit(0);
        }

#ifdef DEBUG
     printf("error = %d\n", err);
#endif

     sread = ssl_read_timeout(sock, ssl, headbuf, recv_size, RECV_TIMEOUT);

        if (sread <= 0 ) {
            close(sock);
/*          fprintf(stdout, "%s - ssl read time out \n", scanhost); */
        exit(0);
        }

#ifdef DEBUG
     printf("sread  = %d \n", sread);
#endif

    }
    else {
    send(sock, sendbuf, len, 0);
    nb = recvtimeout_r(sock, headbuf, recv_size, RECV_TIMEOUT);
    }

    DPRINTF("[script] %s - receive buf \n%s", rscanhost, headbuf); 
    return (rb_str_new(headbuf, recv_size)); 
}

static VALUE
ariclose(VALUE self)
{
    DPRINTF("[script] %s - ariclose \n", rscanhost);
    close(sock);
    return self;    
}


void
rubyscan(char *scanhost, unsigned int port, short int option)
{

    ID ari_send_ID;
    ID arirang_ID;

    VALUE ari_port;
    VALUE ari_recv;
    VALUE ari_recvflag;
    VALUE rhost;
    VALUE ari_sport;
    char destport[5];
   
    if (!proxy) {
        if ((he = gethostbyname(scanhost)) == NULL) {
            perror("host not found");
            /* close(sock);  */
            exit(0);
        }
    }

    snprintf(rscanhost, sizeof(rscanhost), "%s", scanhost);

    signal(SIGINT, breakint);
    /*ariruby_init(ariruby); */

    arirang_ID = rb_intern("Arirang");
    ari_send_ID = rb_intern("asadal");
    /*ariruby_run(); */

    rb_cArirang = rb_const_get(rb_cObject, arirang_ID);
    rb_arirang = rb_class_new_instance(0, NULL, rb_cArirang);
  
    rb_define_method(rb_cArirang, "ariconnect", ariconnect, 0);
    rb_define_method(rb_cArirang, "arisend", arisend, 1);
    rb_define_method(rb_cArirang, "ariclose", ariclose, 0);
    rb_define_method(rb_cArirang, "ariprint", ariprint, 1);

    ari_port = rb_ivar_get(rb_cArirang, rb_intern("@ari_port"));
    ari_recv = rb_ivar_get(rb_cArirang, rb_intern("@ari_recvsize"));
    ari_recvflag = rb_ivar_get(rb_cArirang, rb_intern("@ari_recvflag"));

/*
    ari_hash = rb_hash_aref(ari_info, rb_str_new2("description"));
    ari_array = rb_ary_entry(ari_info, 0);
*/

    if (ari_port != Qnil)
        port = NUM2INT(ari_port);

    if (proxy) {
        rport = port;

#ifdef DEBUG
        printf("rscanhost = %s, rport = %d \n", rscanhost, rport);
#endif

    }

    if (ari_recv != Qnil)  
        recv_size = NUM2INT(ari_recv);

    if (ari_recvflag != Qnil)
        recv_flag = NUM2INT(ari_recvflag);

    switch (recv_flag) {
        case 1:
            recv_flag = MSG_OOB;
            break;
        case 2:
            recv_flag = MSG_PEEK;
            break;
        case 3:
            recv_flag = MSG_WAITALL;
            break;
        case 4:
            recv_flag = MSG_DONTWAIT;
            break;
        default:
            recv_flag = 0;
    }

    if (!proxy) {
        http.sin_family = AF_INET;
        http.sin_port = htons(port);
        http.sin_addr = *((struct in_addr *) he->h_addr);
        bzero(&(http.sin_zero), 8);
    }

    rhost = rb_str_new2(scanhost);
    rb_define_variable("$ari_host", &rhost);
    snprintf(destport, sizeof(destport), "%d", port);
    ari_sport = rb_str_new2(destport);
    rb_define_variable("$ari_sport", &ari_sport);

    DPRINTF("[script] load asadal - %s\n", rscanhost);
    rb_funcall(rb_arirang, ari_send_ID, 0);

    if (sslflags) {
        if (err <= 0) {
        close(sock);
        fprintf(stdout, "%s - ssl write time out \n", scanhost);
        exit(0);
        }

    ssl_error(ssl, err, scanhost);

#ifdef DEBUG
    printf("error = %d\n", err);
#endif

    }

    if (sslflags) {
        if (sread <= 0 ) {
            fprintf(stdout, "%s - ssl read time out \n", scanhost);
            close(sock);
            exit(0);
        }
    }
   
    if (sendbuf != NULL)
        free(sendbuf);

    if (headbuf != NULL)
        free(headbuf);

    if (nb == -1 || nb == -2 ) {
        if (nb == -2)
            fprintf(stdout, "%s - %d/tcp - open port but no receive \n", scanhost, port); 

        close(sock);
        if (!pflags)
           exit(0);
    }

    if (sslflags)
        ssl_free(ctx, ssl);

    ariruby_exit();

    close(sock);

}


void
rubyscanW(unsigned long host, unsigned int port, unsigned short int type, short int option)
{

    ID ari_send_ID;
    ID arirang_ID;

    VALUE ari_port;
    VALUE ari_recv;
    VALUE ari_recvflag;
    VALUE rhost;
    VALUE ari_sport;
    char destport[5];

    signal(SIGINT, breakint);

    /*ariruby_init(ariruby);*/

    arirang_ID = rb_intern("Arirang");
    ari_send_ID = rb_intern( "asadal" );

    /*ariruby_run();*/

    rb_cArirang = rb_const_get(rb_cObject, arirang_ID);
    rb_arirang = rb_class_new_instance(0, NULL, rb_cArirang);

    rb_define_method(rb_cArirang, "ariconnect", ariconnect, 0);
    rb_define_method(rb_cArirang, "arisend", arisend, 1);
    rb_define_method(rb_cArirang, "ariclose", ariclose, 0);
    rb_define_method(rb_cArirang, "ariprint", ariprint, 1);

    ari_port = rb_ivar_get(rb_cArirang, rb_intern("@ari_port"));
    ari_recv = rb_ivar_get(rb_cArirang, rb_intern("@ari_recvsize"));
    ari_recvflag = rb_ivar_get(rb_cArirang, rb_intern("@ari_recvflag"));
/*
    ari_hash = rb_hash_aref(ari_info, rb_str_new2("description"));
    ari_array = rb_ary_entry(ari_info, 0);
*/
    
    if (ari_port != Qnil)
        port = NUM2INT(ari_port);

    if (ari_recv != Qnil)  
        recv_size = NUM2INT(ari_recv);

    if (ari_recvflag != Qnil)
        recv_flag = NUM2INT(ari_recvflag);

    switch (recv_flag) {
        case 1:
            recv_flag = MSG_OOB;
            break;
        case 2:
            recv_flag = MSG_PEEK;
            break;
        case 3:
            recv_flag = MSG_WAITALL;
            break;
        case 4:
            recv_flag = MSG_DONTWAIT;
            break;
        default:
            recv_flag = 0;
    }

    http.sin_family = AF_INET;
    http.sin_port = htons(port);
    http.sin_addr.s_addr = htonl(host);

    snprintf(rscanhost, sizeof(rscanhost), "%s", inet_ntoa(http.sin_addr));

    if (proxy) {
        rport = port;
#ifdef DEBUG
    printf("rscanhost = %s, rport = %d \n", rscanhost, rport);
#endif
    }

    rhost = rb_str_new2(inet_ntoa(http.sin_addr));
    rb_define_variable("$ari_host", &rhost);
    snprintf(destport, sizeof(destport), "%d", port);
    ari_sport = rb_str_new2(destport);
    rb_define_variable("$ari_sport", &ari_sport);

    DPRINTF("[script] load asadal - %s\n", rscanhost);
    rb_funcall(rb_arirang, ari_send_ID, 0);

    if (sslflags) {
        if (err <= 0) {
        close(sock);
        fprintf(stdout, "%s - ssl write time out \n", inet_ntoa(http.sin_addr));
        exit(0);
        }

    ssl_error(ssl, err, inet_ntoa(http.sin_addr)); 
#ifdef DEBUG
    printf("error = %d\n", err); 
#endif
    }

    if (sslflags) {
        if (sread <= 0 ) {
            fprintf(stdout, "%s - ssl read time out \n", inet_ntoa(http.sin_addr));
            close(sock);
            exit(0);
        }
    }
#ifdef DEBUG
    printf("sread  = %d, buf size= %d \n", sread, strlen(headbuf)); 
#endif

    if (sendbuf != NULL)
        free(sendbuf);

    if (nb == -1 || nb == -2) { 
#if !(defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__))
        alarm(0);
#endif
        if (nb == -2)
            fprintf(stdout, "%s - %d/tcp - open port but no receive \n", inet_ntoa(http.sin_addr), port);
    
        close(sock);
        if (!pflags)
            exit(0);
    }

    if (strstr(headbuf, "Server Error") != NULL) {
#if !(defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__))
        alarm(0);
#endif
        fprintf(stdout, "%s - %d/tcp - server error \n", inet_ntoa(http.sin_addr), port);
        close(sock);
        if (!pflags)
            exit(0);
    }

    if (headbuf != NULL)
        free(headbuf);

    if (sslflags)
        ssl_free(ctx, ssl);

    ariruby_exit();
    close(sock);
}
